@@ -4,9 +4,10 @@ import datetime
 from collections import deque, defaultdict
 from heapq import heappop, heappush
 from trytond.model import ModelView, ModelSQL, fields
-from trytond.wizard import Wizard
-from trytond.pyson import Eval, Not, Equal
+from trytond.wizard import Wizard, StateTransition
+from trytond.pyson import Eval
 from trytond.transaction import Transaction
+from trytond.pool import Pool
 
 
 def intfloor(x):
@@ -17,22 +18,24 @@ class Work(ModelSQL, ModelView):
     _name = 'project.work'
 
     predecessors = fields.Many2Many('project.predecessor_successor',
-            'successor', 'predecessor', 'Predecessors',
-            domain=[
-                ('parent', '=', Eval('parent')),
-                ('id', '!=', Eval('active_id')),
-            ])
+        'successor', 'predecessor', 'Predecessors',
+        domain=[
+            ('parent', '=', Eval('parent')),
+            ('id', '!=', Eval('id')),
+            ],
+        depends=['parent', 'id'])
     successors = fields.Many2Many('project.predecessor_successor',
-            'predecessor', 'successor', 'Successors',
-            domain=[
-                ('parent', '=', Eval('parent')),
-                ('id', '!=', Eval('active_id')),
-            ])
+        'predecessor', 'successor', 'Successors',
+        domain=[
+            ('parent', '=', Eval('parent')),
+            ('id', '!=', Eval('id')),
+            ],
+        depends=['parent', 'id'])
     leveling_delay = fields.Float("Leveling Delay")
     back_leveling_delay = fields.Float("Leveling Delay")
     allocations = fields.One2Many('project.allocation', 'work', 'Allocations',
-            states={
-                'invisible': Not(Equal(Eval('type'), 'task')),
+        states={
+            'invisible': Eval('type') != 'task',
             }, depends=['type'])
     duration = fields.Function(fields.Float('Duration'), 'get_function_fields')
     early_start_time = fields.DateTime("Early Start Time", readonly=True)
@@ -86,7 +89,7 @@ class Work(ModelSQL, ModelView):
         :return: a dictionary with all field names as key and
             a dictionary as value with id as key
         '''
-        req_ref_obj = self.pool.get('res.request.reference')
+        req_ref_obj = Pool().get('res.request.reference')
 
         cursor = Transaction().cursor
 
@@ -147,7 +150,6 @@ class Work(ModelSQL, ModelView):
                 leafs = parents
             res['duration'] = durations
 
-
         fun_fields = ('early_start_date', 'early_finish_date',
                       'late_start_date', 'late_finish_date',
                       'actual_start_date', 'actual_finish_date',
@@ -171,33 +173,49 @@ class Work(ModelSQL, ModelView):
         return res
 
     def set_function_fields(self, ids, name, value):
-        request_obj = self.pool.get('res.request')
-        req_ref_obj = self.pool.get('res.request.reference')
+        request_obj = Pool().get('res.request')
+        req_ref_obj = Pool().get('res.request.reference')
 
         if name == 'requests':
             works = self.browse(ids)
             currents = dict((req.id, req) for work in works for req in
                     work.requests)
+            if not value:
+                return
             for v in value:
                 to_unlink = []
                 to_link = []
                 operator = v[0]
 
                 target_ids = len(v) > 1 and v[1] or []
-                if operator == 'set':
+                if operator == 'create':
+                    to_link.append(request_obj.create(v[1]))
+                elif operator == 'write':
+                    request_obj.write(v[1], v[2])
+                elif operator == 'delete':
+                    request_obj.delete(v[1])
+                elif operator == 'delete_all':
+                    target_ids = []
+                    for record_id in ids:
+                        ref_ids = req_ref_obj.search([
+                                ('reference', '=',
+                                    'project.work,%s' % record_id),
+                                ])
+                        refs = req_ref_obj.browse(ref_ids)
+                        target_ids.extend(ref.request.id for ref in refs)
+                    request_obj.delete(target_ids)
+                elif operator == 'unlink':
+                    to_unlink.extend((i for i in target_ids if i in currents))
+                elif operator == 'add':
+                    to_link.extend((i for i in target_ids if i not in
+                        currents))
+                elif operator == 'unlink_all':
+                    to_unlink.extend(currents)
+                elif operator == 'set':
                     to_link.extend((i for i in target_ids
                         if i not in currents))
                     to_unlink.extend((i for i in currents
                         if i not in target_ids))
-                elif operator == 'add':
-                    to_link.extend((i for i in target_ids if i not in
-                        currents))
-                elif operator == 'unlink':
-                    to_unlink.extend((i for i in target_ids if i in currents))
-                elif operator == 'unlink_all':
-                    to_unlink.extend(currents)
-                elif operator == 'delete':
-                    request_obj.delete(to_delete)
                 else:
                     raise Exception('Operation not supported')
 
@@ -227,7 +245,7 @@ class Work(ModelSQL, ModelView):
                         db_field: value \
                                 and datetime.datetime.combine(value,
                                     datetime.time()) \
-                                or False,
+                                or None,
                         })
                 break
 
@@ -252,10 +270,8 @@ class Work(ModelSQL, ModelView):
         return date
 
     def add_hours(self, company, date, hours):
-        day_per_week = company.hours_per_work_week / company.hours_per_work_day
-
         while hours:
-            if hours !=  intfloor(hours):
+            if hours != intfloor(hours):
                 minutes = (hours - intfloor(hours)) * 60
                 date = self.add_minutes(company, date, minutes)
             hours = intfloor(hours)
@@ -283,7 +299,7 @@ class Work(ModelSQL, ModelView):
         day_per_week = company.hours_per_work_week / company.hours_per_work_day
 
         while days:
-            if days !=  intfloor(days):
+            if days != intfloor(days):
                 hours = (days - intfloor(days)) * company.hours_per_work_day
                 date = self.add_hours(company, date, hours)
             days = intfloor(days)
@@ -296,7 +312,7 @@ class Work(ModelSQL, ModelView):
             if weeks:
                 date = self.add_weeks(company, date, weeks)
 
-            date += datetime.timedelta(days= -date.weekday() + intfloor(days))
+            date += datetime.timedelta(days=-date.weekday() + intfloor(days))
 
             days = days - intfloor(days)
 
@@ -305,12 +321,12 @@ class Work(ModelSQL, ModelView):
     def add_weeks(self, company, date, weeks):
         day_per_week = company.hours_per_work_week / company.hours_per_work_day
 
-        if weeks !=  intfloor(weeks):
+        if weeks != intfloor(weeks):
             days = (weeks - intfloor(weeks)) * day_per_week
             if days:
                 date = self.add_days(company, date, days)
 
-        date += datetime.timedelta(days= 7 * intfloor(weeks))
+        date += datetime.timedelta(days=7 * intfloor(weeks))
 
         return date
 
@@ -321,9 +337,8 @@ class Work(ModelSQL, ModelView):
             'early_finish_time', work['early_finish_time'])
         get_late_start = lambda work: values.get(work, {}).get(
             'late_start_time', work['late_start_time'])
-        maxdate = lambda x,y: x and y and max(x,y) or x or y
-        mindate = lambda x,y: x and y and min(x,y) or x or y
-
+        maxdate = lambda x, y: x and y and max(x, y) or x or y
+        mindate = lambda x, y: x and y and min(x, y) or x or y
 
         # propagate constraint_start_time
         constraint_start = reduce(maxdate, (pred.early_finish_time \
@@ -504,7 +519,7 @@ class Work(ModelSQL, ModelView):
                              set(s.id for s in w.successors))
 
         work = self.browse(work_id)
-        parent_id = work.parent and work.parent.id or False
+        parent_id = work.parent and work.parent.id or None
         sibling_ids = self.search([
                 ('parent', '=', parent_id)
                 ])
@@ -528,11 +543,13 @@ class Work(ModelSQL, ModelView):
         # define some helper functions
         get_key = lambda w: (set(p.id for p in w.predecessors),
                              set(s.id for s in w.successors))
-        over_alloc = lambda current_alloc, work: \
-            reduce(lambda res, alloc: res or current_alloc[alloc.employee.id] + \
-                       alloc.percentage > 100,
-                   work.allocations,
-                   False)
+        over_alloc = lambda current_alloc, work: (
+            reduce(lambda res, alloc: (res
+                    or (current_alloc[alloc.employee.id]
+                        + alloc.percentage) > 100),
+                work.allocations,
+                False))
+
         def sum_allocs(current_alloc, work):
             res = defaultdict(float)
             for alloc in work.allocations:
@@ -579,13 +596,14 @@ class Work(ModelSQL, ModelView):
                 yield sibling, delay
 
         work = self.browse(work_id)
-        parent_id = work.parent and work.parent.id or False
+        parent_id = work.parent and work.parent.id or None
         sibling_ids = self.search([
                 ('parent', '=', parent_id)
                 ])
 
         refkey = get_key(work)
-        siblings = [s for s in self.browse(sibling_ids) if get_key(s) == refkey]
+        siblings = [s for s in self.browse(sibling_ids)
+            if get_key(s) == refkey]
 
         for sibling, delay in compute_delays(siblings):
             self.write(sibling.id, {
@@ -611,7 +629,7 @@ class Work(ModelSQL, ModelView):
                 self.reset_leveling(work_id)
         fields = ('constraint_start_time', 'constraint_finish_time',
                   'effort')
-        if reduce(lambda x, y : x or y in values, fields, False):
+        if reduce(lambda x, y: x or y in values, fields, False):
             for work_id in ids:
                 self.compute_dates(work_id)
         return res
@@ -642,19 +660,18 @@ class Work(ModelSQL, ModelView):
 Work()
 
 
-
 class PredecessorSuccessor(ModelSQL):
     'Predecessor - Successor'
     _name = 'project.predecessor_successor'
     _description = __doc__
 
     predecessor = fields.Many2One('project.work', 'Predecessor',
-            ondelete='CASCADE', required=True, select=1)
+            ondelete='CASCADE', required=True, select=True)
     successor = fields.Many2One('project.work', 'Successor',
-            ondelete='CASCADE', required=True, select=1)
+            ondelete='CASCADE', required=True, select=True)
 
     def write(self, ids, values):
-        work_obj = self.pool.get('project.work')
+        work_obj = Pool().get('project.work')
         res = super(PredecessorSuccessor, self).write(ids, values)
 
         for work_id in values.itervalues():
@@ -664,7 +681,7 @@ class PredecessorSuccessor(ModelSQL):
         return res
 
     def delete(self, ids):
-        work_obj = self.pool.get('project.work')
+        work_obj = Pool().get('project.work')
         if isinstance(ids, (int, long)):
             ids = [ids]
 
@@ -689,7 +706,7 @@ class PredecessorSuccessor(ModelSQL):
         return res
 
     def create(self, values):
-        work_obj = self.pool.get('project.work')
+        work_obj = Pool().get('project.work')
         ps_id = super(PredecessorSuccessor, self).create(values)
 
         pred_succ = self.browse(ps_id)
@@ -706,20 +723,12 @@ PredecessorSuccessor()
 class Leveling(Wizard):
     'Tasks Leveling'
     _name = 'project_plan.work.leveling'
+    start_state = 'leveling'
+    leveling = StateTransition()
 
-    states = {
-        'init': {
-            'result': {
-                'type': 'action',
-                'action': '_leveling',
-                'state': 'end',
-                },
-            },
-        }
-
-    def _leveling(self, data):
-        work_obj = self.pool.get('project.work')
-        work_obj.create_leveling(data['id'])
-        return {}
+    def transition_leveling(self, session):
+        work_obj = Pool().get('project.work')
+        work_obj.create_leveling(Transaction().context['active_id'])
+        return 'end'
 
 Leveling()
